﻿using System;
using System.Drawing;
using System.Windows.Forms;

using Steema.TeeChart.Drawing;
using Steema.TeeChart.Styles;
using Steema.TeeChart.Tools;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 페이지 번호
        /// </summary>
        private PageNumber pageNumber;

        /// <summary>
        /// 포인트 시리즈
        /// </summary>
        private Points points;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            Text = "Points 클래스 : XValues/YValues/FirstVisibleIndex/LastVisibleIndex 속성을 사용해 현재 페이지의 최소/최대값 찾기";

            this.adjustVerticalAxisCheckBox.Checked = true;

            this.tChart.Panel.Pen             = new ChartPen(Color.Black);
            this.tChart.Page.MaxPointsPerPage = 25;

            this.pageNumber = new PageNumber(this.tChart.Chart);

            this.points = new Points(this.tChart.Chart);

            this.points.FillSampleValues(100);

            this.adjustVerticalAxisCheckBox.CheckedChanged += adjustVerticalAxisCheckBox_CheckedChanged;
            this.previousButton.Click                      += previousButton_Click;
            this.nextButton.Click                          += nextButton_Click;

            AdjustAxisScale(this.points, 0, 24, true);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 수직 축 조정 체크 박스 체크 변경시 처리하기 - adjustVerticalAxisCheckBox_CheckedChanged(sender, e)

        /// <summary>
        /// 수직 축 조정 체크 박스 체크 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void adjustVerticalAxisCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if(this.adjustVerticalAxisCheckBox.Checked)
            {
                AdjustAxisScale(this.points, this.points.FirstVisibleIndex, this.points.LastVisibleIndex, true);
            }
            else
            {
                this.points.GetVertAxis.Automatic = true;
            }

            this.points.Repaint();
        }

        #endregion
        #region 이전 버튼 클릭시 처리하기 - previousButton_Click(sender, e)

        /// <summary>
        /// 이전 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void previousButton_Click(object sender, EventArgs e)
        {
            this.tChart.Page.Previous();

            if(this.adjustVerticalAxisCheckBox.Checked)
            {
                AdjustAxisScale(this.points, this.points.FirstVisibleIndex, this.points.LastVisibleIndex, true);
            }
            else
            {
                this.points.GetVertAxis.Automatic = true;
            }
        }

        #endregion
        #region 다음 버튼 클릭시 처리하기 - nextButton_Click(sender, e)

        /// <summary>
        /// 다음 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void nextButton_Click(object sender, EventArgs e)
        {
            this.tChart.Page.Next();

            if(this.adjustVerticalAxisCheckBox.Checked)
            {
                AdjustAxisScale(this.points, this.points.FirstVisibleIndex, this.points.LastVisibleIndex, true);
            }
            else
            {
                this.points.GetVertAxis.Automatic = true;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 최소/최대값 찾기 - FindMinimumMaximum(valueList, firstIndex, lastIndex, minimum, maximum)

        /// <summary>
        /// 최소/최대값 찾기
        /// </summary>
        /// <param name="valueList">값 리스트</param>
        /// <param name="firstIndex">첫번째 인덱스</param>
        /// <param name="lastIndex">마지막 인덱스</param>
        /// <param name="minimum">최소값</param>
        /// <param name="maximum">최대값</param>
        private void FindMinimumMaximum(ValueList valueList, int firstIndex, int lastIndex, out double minimum, out double maximum)
        {
            minimum = valueList[firstIndex];
            maximum = valueList[lastIndex ];

            for(int i = firstIndex; i <= lastIndex; i++)
            {
                if(valueList.Value[i] < minimum)
                {
                    minimum = valueList[i];
                }
                else if(valueList.Value[i] > maximum)
                {
                    maximum = valueList[i];
                }
            }
        }

        #endregion
        #region 축 스케일 조정하기 - AdjustAxisScale(points, firstIndex, lastIndex, vertical)

        /// <summary>
        /// 축 스케일 조정하기
        /// </summary>
        /// <param name="points">포인트 시리즈</param>
        /// <param name="firstIndex">첫번째 인덱스</param>
        /// <param name="lastIndex">마지막 인덱스</param>
        /// <param name="vertical">수직 여부</param>
        private void AdjustAxisScale(Points points, int firstIndex, int lastIndex, bool vertical)
        {
            double minimum;
            double maximum;

            if(vertical)
            {
                FindMinimumMaximum(points.YValues, firstIndex, lastIndex, out minimum, out maximum);

                points.GetVertAxis.Automatic = false;

                points.GetVertAxis.SetMinMax(minimum, maximum);
            }
            else
            {
                FindMinimumMaximum(points.XValues, firstIndex, lastIndex, out minimum, out maximum);

                points.GetHorizAxis.Automatic = false;

                points.GetHorizAxis.SetMinMax(minimum, maximum);
            }
        }

        #endregion
    }
}